#!/usr/bin/perl -w

use strict;
use Getopt::Long;
use DBI;
use Text::Wrap;
use Shell qw(basename); # Assuming this script is going to run on Linux
Getopt::Long::Configure ("posix_default", "no_ignore_case");

use Env qw(LSBUSER LSBDBPASSWD LSBDB LSBDBHOST);

# Options
my $DBName = $LSBDB;
my $DBUser = $LSBUSER;
my $DBPass = $LSBDBPASSWD;
my $DBHost = $LSBDBHOST;

if(@ARGV != 1) {
    print "Usage: $0 distr_id\n";
    exit 1;
}

my $dbh;
if( $DBPass ) {
        $dbh = DBI->connect('DBI:mysql:database='.$DBName.';host='.$DBHost, $DBUser, $DBPass)
        or die "Couldn't connect to database: " . DBI->errstr;
}
else {
        $dbh = DBI->connect('DBI:mysql:database='.$DBName.';host='.$DBHost, $DBUser)
        or die "Couldn't connect to database: " . DBI->errstr;
}

my $Did=0;

if( $ARGV[0] =~ /\d+/ ) {
    $Did = $ARGV[0];
}
else {
    die "Please specify valid id";
}


$dbh->do("delete from Distribution where Did=$Did");

my $comps_q = $dbh->prepare("SELECT Cid FROM Component WHERE Cdistr=$Did");
$comps_q->execute();
while( my($Cid) = $comps_q->fetchrow_array() ) {
    my ($Calias) = $dbh->selectrow_array("SELECT Cid FROM Component WHERE Calias=$Cid AND Cdistr != $Did LIMIT 1");
    if( $Calias ) {
        $dbh->do("UPDATE Component SET Calias=$Calias WHERE Calias=$Cid");
    	$dbh->do("UPDATE Component SET Calias=0 WHERE Calias=Cid");
        $dbh->do("UPDATE RawCommand SET RCcomponent=$Calias WHERE RCcomponent=$Cid");
        $dbh->do("UPDATE RawLibrary SET RLcomponent=$Calias WHERE RLcomponent=$Cid");
        $dbh->do("UPDATE CompRILM SET CRMcid=$Calias WHERE CRMcid=$Cid");
#        $dbh->do("UPDATE CompJInt SET CJIcid=$Calias WHERE CJIcid=$Cid");
    }
    else {
        $dbh->do("DROP TABLE IF EXISTS tmp_Libs");
        $dbh->do("CREATE TEMPORARY TABLE tmp_Libs (KEY k_RLid(RLid)) SELECT RLid FROM RawLibrary WHERE RLcomponent=$Cid");
        my ($lib_exists) = $dbh->selectrow_array("SELECT 1 FROM tmp_Libs LIMIT 1");
        if( $lib_exists ) {
            $dbh->do("DELETE FROM RawLibrary WHERE RLcomponent=$Cid");
	    $dbh->do("DELETE FROM RawCommand WHERE RCcomponent=$Cid");
            $dbh->do("DELETE FROM CompRILM WHERE CRMcid=$Cid");
#   	     $dbh->do("DELETE FROM CompJInt WHERE CJIcid=$Cid");
            $dbh->do("DELETE FROM RLibRInt WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=RLRIrlid)");
	    $dbh->do("DELETE FROM WeakSymbol WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=WSrlid)");
	    $dbh->do("DELETE FROM CompatSymbol WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=CSrlid)");
	}
    }

    $dbh->do("delete from Component where Cid=$Cid");
}

$dbh->do("delete from RLibDeps where RLDrlid not in (select RLid from RawLibrary)");
$dbh->do("delete from RLibRClass where RLRCrlid not in(select RLid from RawLibrary)");
$dbh->do("delete from CompLDpath where CLDcid not in (select Cid from Component)");
$dbh->do("delete from CompRILM where CRMcid not in (select Cid from Component)");
$dbh->do("delete from RLibLink where RLLrlid not in (select RLid from RawLibrary)");

print "\n\n!!!!! REGENERATE CACHE TABLES MANUALLY !!!!!\n\n";

