#!/usr/bin/perl

use DBI;
use Env qw(LSBUSER LSBDBPASSWD LSBDB LSBDBHOST);
use strict;

my $quiet=0;

my $DBName = $LSBDB;
my $DBUser = $LSBUSER;
my $DBPass = $LSBDBPASSWD;
my $DBHost = $LSBDBHOST;

if(@ARGV != 1) {
    die "One argument (Distribution id) is required";
}

my $Cdistr=$ARGV[0];

my $dbh = DBI->connect('DBI:mysql:database='.$DBName.';host='.$DBHost, $DBUser, $DBPass)
    or die "Couldn't connect to database: " . DBI->errstr;

my %found_aliases;

# Temporary tables with number of different objects in every component
# Note that we don't look at classes here, since if two components
# have the same sets of libraries and interfaces, then they automatically
# have the same sets of classes
$dbh->do("CREATE TEMPORARY TABLE tmp_Cmds (KEY Cid(Cid))
            SELECT RCcomponent AS Cid, COUNT(RCid) AS cmd_cnt FROM RawCommand
            GROUP BY Cid");

$dbh->do("CREATE TEMPORARY TABLE tmp_Classes (KEY Cid(Cid))
            SELECT RLcomponent AS Cid, (
                SELECT COUNT(RLRCrcid) FROM RLibRClass WHERE RLid=RLRCrlid
            ) AS class_cnt
            FROM RawLibrary");

$dbh->do("CREATE TEMPORARY TABLE tmp_RLibContent (KEY RLid(RLRIrlid))
            SELECT RLRIrlid, COUNT(RLRIriid) AS int_cnt FROM RLibRInt
            GROUP BY RLRIrlid");

$dbh->do("CREATE TEMPORARY TABLE tmp_Ints (KEY Cid(Cid))
            SELECT RLcomponent AS Cid, COUNT(RLRIrlid) AS lib_cnt, int_cnt FROM RawLibrary
            JOIN tmp_RLibContent ON RLid=RLRIrlid
            GROUP BY Cid");

$dbh->do("CREATE TEMPORARY TABLE tmp_RILMs (KEY Cid(Cid))
            SELECT CRMcid AS Cid, COUNT(CRMrilmid) AS rilm_cnt FROM CompRILM
            GROUP BY Cid");

#$dbh->do("CREATE TEMPORARY TABLE tmp_Jints (KEY Cid(Cid))
#            SELECT CJIcid AS Cid, COUNT(CJIjiid) AS jint_cnt FROM CompJInt
#            GROUP BY Cid");

$dbh->do("CREATE TEMPORARY TABLE tmp_DistrCompContent (KEY Cid(lib_cnt,class_cnt,int_cnt,rilm_cnt,cmd_cnt, Cid),
                                                       KEY class_cnt(class_cnt),
                                                       KEY int_cnt(int_cnt),
                                                       KEY rilm_cnt(rilm_cnt),
                                                       KEY cmd_cnt(cmd_cnt)
                                                        )
            SELECT lib_cnt, class_cnt, int_cnt, rilm_cnt, cmd_cnt, Cid FROM Component
            LEFT JOIN tmp_Cmds USING(Cid)
            LEFT JOIN tmp_Classes USING(Cid)
            LEFT JOIN tmp_Ints USING(Cid)
            LEFT JOIN tmp_RILMs USING(CID)
            WHERE Cdistr=$Cdistr
            AND Calias=0
            ");

$dbh->do("CREATE TEMPORARY TABLE tmp_OtherCompContent (KEY Cid(lib_cnt,class_cnt,int_cnt,rilm_cnt,cmd_cnt, Cid),
                                                       KEY class_cnt(class_cnt),
                                                       KEY int_cnt(int_cnt),
                                                       KEY rilm_cnt(rilm_cnt),
                                                       KEY cmd_cnt(cmd_cnt),
                                                       KEY Cid2(Cid)
                                                        )
            SELECT lib_cnt, class_cnt, int_cnt, rilm_cnt, cmd_cnt, Cid, Cdistr FROM Component
            LEFT JOIN tmp_Cmds USING(Cid)
            LEFT JOIN tmp_Classes USING(Cid)
            LEFT JOIN tmp_Ints USING(Cid)
            LEFT JOIN tmp_RILMs USING(CID)
            WHERE Calias=0
            ");

$dbh->do("UPDATE tmp_OtherCompContent SET lib_cnt=0 WHERE lib_cnt IS NULL");
$dbh->do("UPDATE tmp_OtherCompContent SET cmd_cnt=0 WHERE cmd_cnt IS NULL");
$dbh->do("UPDATE tmp_OtherCompContent SET class_cnt=0 WHERE class_cnt IS NULL");
$dbh->do("UPDATE tmp_OtherCompContent SET int_cnt=0 WHERE int_cnt IS NULL");
$dbh->do("UPDATE tmp_OtherCompContent SET rilm_cnt=0 WHERE rilm_cnt IS NULL");
$dbh->do("UPDATE tmp_DistrCompContent SET lib_cnt=0 WHERE lib_cnt IS NULL");
$dbh->do("UPDATE tmp_DistrCompContent SET cmd_cnt=0 WHERE cmd_cnt IS NULL");
$dbh->do("UPDATE tmp_DistrCompContent SET class_cnt=0 WHERE class_cnt IS NULL");
$dbh->do("UPDATE tmp_DistrCompContent SET int_cnt=0 WHERE int_cnt IS NULL");
$dbh->do("UPDATE tmp_DistrCompContent SET rilm_cnt=0 WHERE rilm_cnt IS NULL");

my $libs_Q = $dbh->prepare( 'SELECT RLname, RLrunname, RLpath, RLversion, RLsoname FROM RawLibrary WHERE RLcomponent = ? UNION DISTINCT SELECT RLname, RLrunname, RLpath, RLversion, RLsoname FROM RawLibrary WHERE RLcomponent = ?' );
my $single_lib_Q = $dbh->prepare( 'SELECT RLid FROM RawLibrary WHERE RLname=? AND RLrunname=? AND RLpath=? AND RLversion=? AND RLcomponent=? AND RLsoname=?' );

my $cmds_Q = $dbh->prepare( 'SELECT RCname, RCpath, RCcomment FROM RawCommand WHERE RCcomponent = ? UNION DISTINCT SELECT RCname, RCpath, RCcomment FROM RawCommand WHERE RCcomponent = ?' );
my $single_cmd_nocomment_Q = $dbh->prepare( 'SELECT RCid FROM RawCommand WHERE RCname=? AND RCpath=? AND RCcomponent=?' );
my $single_cmd_Q = $dbh->prepare( 'SELECT RCid FROM RawCommand WHERE RCname=? AND RCpath=? AND RCcomment=? AND RCcomponent=?' );

my $rilms_Q = $dbh->prepare( 'SELECT C1.CRMcid FROM CompRILM C1 WHERE C1.CRMcid=? AND NOT EXISTS (SELECT 1 FROM CompRILM C2 WHERE C2.CRMcid=? AND C2.CRMrilmid=C1.CRMrilmid)' );
my $rints_Q = $dbh->prepare( 'SELECT 1 FROM RLibRInt R1 WHERE R1.RLRIrlid=? AND NOT EXISTS (SELECT 1 FROM RLibRInt R2 WHERE R2.RLRIrlid=? AND R2.RLRIriid=R1.RLRIriid) LIMIT 1' );
my $rldeps_Q = $dbh->prepare( 'SELECT 1 FROM RLibDeps R1 WHERE R1.RLDrlid=? AND NOT EXISTS (SELECT 1 FROM RLibDeps R2 WHERE R2.RLDrlid=? AND R2.RLDrlsid=R1.RLDrlsid) LIMIT 1' );
my $rllinks_Q = $dbh->prepare( 'SELECT 1 FROM RLibLink R1 WHERE R1.RLLrlid=? AND NOT EXISTS (SELECT 1 FROM RLibLink R2 WHERE R2.RLLrlid=? AND R2.RLLlinkname=R1.RLLlinkname) LIMIT 1' );

my $ldpaths_Q = $dbh->prepare( 'SELECT 1 FROM CompLDpath C1 WHERE C1.CLDcid=? AND NOT EXISTS (SELECT 1 FROM CompLDpath C2 WHERE C2.CLDcid=? AND C2.CLDcid=C1.CLDcid) LIMIT 1' );

my $other_comps_Q = $dbh->prepare( "SELECT DISTINCT Cid FROM tmp_OtherCompContent
                                    WHERE lib_cnt = ?
                                    AND class_cnt = ?
                                    AND int_cnt = ?
                                    AND rilm_cnt = ?
                                    AND cmd_cnt = ?
                                    AND (Cdistr != $Cdistr OR Cid > ?)" );

my $comps_Q = $dbh->prepare( 'SELECT Cid, lib_cnt, class_cnt, int_cnt, rilm_cnt, cmd_cnt FROM tmp_DistrCompContent ORDER BY Cid' );
$comps_Q->execute();

print "CREATE TEMPORARY TABLE tmp_Libs(RLid INT(10) UNSIGNED, KEY RLid(RLid));\n";

while( my @comp_row = $comps_Q->fetchrow_array() ) {
    my $cid = $comp_row[0];
    my $lib_cnt = $comp_row[1];
    my $class_cnt = $comp_row[2];
    my $int_cnt = $comp_row[3];
    my $rilm_cnt = $comp_row[4];
    my $cmd_cnt = $comp_row[5];

    $other_comps_Q->execute($lib_cnt, $class_cnt, $int_cnt, $rilm_cnt, $cmd_cnt, $cid) or die();
    while( my @other_comp_row = $other_comps_Q->fetchrow_array() ) {
        my $prev_cid = $other_comp_row[0];

        print STDERR "Possible aliases: $cid and $prev_cid\n";
        my $alias = 1;

        $ldpaths_Q->execute($cid, $prev_cid);
        if( $ldpaths_Q->fetchrow_array() ) {
            $alias = 0;
            print STDERR "Fail_ldpath1: $cid vs $prev_cid\n";
            next;
        }

        $ldpaths_Q->execute($prev_cid, $cid);
        if( $ldpaths_Q->fetchrow_array() ) {
            $alias = 0;
            print STDERR "Fail_ldpath2: $prev_cid vs $cid\n";
            next;
        }

        $libs_Q->execute($prev_cid, $cid);
        while( my @lib_row = $libs_Q->fetchrow_array() ) {
            $single_lib_Q->execute( $lib_row[0], $lib_row[1], $lib_row[2], $lib_row[3], $cid, $lib_row[4] );
            my ($rlid_1) = $single_lib_Q->fetchrow_array();
            if( !$rlid_1 ) {
                $alias = 0;
                print STDERR "Fail_lib1: $lib_row[0], $lib_row[1], $lib_row[2], $lib_row[3], $cid, $lib_row[4]\n";
                last;
            }

            $single_lib_Q->execute( $lib_row[0], $lib_row[1], $lib_row[2], $lib_row[3], $prev_cid, $lib_row[4] );
            my ($rlid_2) = $single_lib_Q->fetchrow_array();
            if( !$rlid_2 ) {
                $alias = 0;
                print STDERR "Fail_lib2: $lib_row[0], $lib_row[1], $lib_row[2], $lib_row[3], $prev_cid, $lib_row[4]\n";
                last;
            }

            $rints_Q->execute($rlid_1, $rlid_2);
            if( $rints_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_ints1: $rlid_1 vs $rlid_2\n";
                last;
            }

            $rints_Q->execute($rlid_2, $rlid_1);
            if( $rints_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_ints2: $rlid_2 vs $rlid_1\n";
                last;
            }

            $rldeps_Q->execute($rlid_1, $rlid_2);
            if( $rldeps_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_deps1: $rlid_1 vs $rlid_2\n";
                last;
            }

            $rldeps_Q->execute($rlid_2, $rlid_1);
            if( $rldeps_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_deps2: $rlid_2 vs $rlid_1\n";
                last;
            }

            $rllinks_Q->execute($rlid_1, $rlid_2);
            if( $rllinks_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_links1: $rlid_1 vs $rlid_2\n";
                last;
            }

            $rllinks_Q->execute($rlid_2, $rlid_1);
            if( $rllinks_Q->fetchrow_array() ) {
                $alias = 0;
                print STDERR "Fail_links2: $rlid_2 vs $rlid_1\n";
                last;
            }
        }

        if( !$alias ) {
            next;
        }

        $cmds_Q->execute($prev_cid, $cid);
        while( my @cmd_row = $cmds_Q->fetchrow_array() ) {
            if( $cmd_row[2] ) {
                $single_cmd_Q->execute( $cmd_row[0], $cmd_row[1], $cmd_row[2], $cid );
                if( !$single_cmd_Q->fetchrow_array() ) {
                    $alias = 0;
                    print STDERR "Fail_cmd1: $cmd_row[0], $cmd_row[1], $cmd_row[2], $cid\n";
                    last;
                }

                $single_cmd_Q->execute( $cmd_row[0], $cmd_row[1], $cmd_row[2], $prev_cid );
                if( !$single_cmd_Q->fetchrow_array() ) {
                    $alias = 0;
                    print STDERR "Fail_cmd2: $cmd_row[0], $cmd_row[1], $cmd_row[2], $prev_cid\n";
                    last;
                }
            }
            else {
                $single_cmd_nocomment_Q->execute( $cmd_row[0], $cmd_row[1], $cid );
                if( !$single_cmd_nocomment_Q->fetchrow_array() ) {
                    $alias = 0;
                    print STDERR "Fail_cmd_1.1: $cmd_row[0], $cmd_row[1], $cid\n";
                    last;
                }

                $single_cmd_nocomment_Q->execute( $cmd_row[0], $cmd_row[1], $prev_cid );
                if( !$single_cmd_nocomment_Q->fetchrow_array() ) {
                    $alias = 0;
                    print STDERR "Fail_cmd_2.1: $cmd_row[0], $cmd_row[1], $prev_cid\n";
                    last;
                }
            }
        }

        if( !$alias ) {
            next;
        }

        # RILMs
        $rilms_Q->execute($prev_cid, $cid);
        if( $rilms_Q->fetchrow_array() ) {
            print STDERR "Fail_rilm1\n";
            next;
        }

        $rilms_Q->execute($cid, $prev_cid);
        if( $rilms_Q->fetchrow_array() ) {
            print STDERR "Fail_rilm2\n";
            next;
        }

        print "INSERT INTO tmp_Libs SELECT RLid FROM RawLibrary WHERE RLcomponent=$cid;\n";

        print "UPDATE Component SET Calias=$prev_cid WHERE Cid=$cid;\n";
        print "UPDATE Component SET Calias=$prev_cid WHERE Calias=$cid;\n";
        print "DELETE FROM RawLibrary WHERE RLcomponent=$cid;\n";
        print "DELETE FROM RawCommand WHERE RCcomponent=$cid;\n";
        print "DELETE FROM CompRILM WHERE CRMcid=$cid;\n";
        print "DELETE FROM CompLDpath WHERE CLDcid=$cid;\n";
        last;
    }
}

print "DELETE FROM RLibDeps WHERE RLDrlid NOT IN (SELECT RLid FROM RawLibrary);\n";
print "DELETE FROM RLibLink WHERE RLLrlid NOT IN (SELECT RLid FROM RawLibrary);\n";
print "DELETE FROM WeakSymbol WHERE WSrlid NOT IN (SELECT RLid FROM RawLibrary);\n";
print "DELETE FROM RLibRClass WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=RLRCrlid);\n";
print "DELETE FROM RLibRInt WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=RLRIrlid);\n";
print "DELETE FROM CompatSymbol WHERE EXISTS (SELECT 1 FROM tmp_Libs WHERE RLid=CSrlid);\n";
