#!/usr/bin/perl

################################################################
# Generate wiki page with the LSB Specification Database schema
# (with empty descriptions for tables and their fields)
#
# Copyright 2008 The Linux Foundation
# Copyright 2008 Institute for System Programming, RAS
#
# Author: Denis Silakov <silakov@ispras.ru>
################################################################

use DBI;
use strict;

# mysql connection parameters are taken from environment variables
use Env qw(LSBUSER LSBDBPASSWD LSBDB LSBDBHOST);

my $dbh = DBI->connect('DBI:mysql:database='.$LSBDB.';host='.$LSBDBHOST, $LSBUSER, $LSBDBPASSWD)
	or die "Couldn't connect to database: ".DBI->errstr;

# Get list of all tables
my $sth = $dbh->prepare("SHOW TABLES") or die DBI->errstr;
$sth->execute or die DBI->errstr;

for(1..$sth->rows) {
	(my $table) = $sth->fetchrow_array;
	
	# Special case for the lsb specdb - skip cache tables
	if( $table =~ /^cache/ ) {
		next;
	}
	
	print "== $table ==\n\n";
	print "{|class=\"wikitable\"\n";
	print "! Field !! Type !! Default !! NULL !! Description\n";

	# Remember primary key members as well as members of unique keys
	my $kth = $dbh->prepare("SHOW KEYS FROM $table") or die DBI->errstr;
	$kth->execute or die DBI->errstr;
	
	my %unique_keys_members = ();
	my %primary_key_members = ();

	for(1..$kth->rows) {
		my $key = $kth->fetchrow_hashref;
		if( $key->{'Key_name'} eq "PRIMARY" ) {
			$primary_key_members{$key->{'Column_name'}} = 1;
		}
		elsif( $key->{'Non_unique'} == 0 ) {
			$unique_keys_members{$key->{'Column_name'}} = 1;
		}
	}

	# Get list of fields
	my $fth = $dbh->prepare("SHOW FIELDS FROM $table") or die DBI->errstr;
	$fth->execute or die DBI->errstr;
	
	for(1..$fth->rows) {
		my $field = $fth->fetchrow_hashref;
		my $name = $field->{'Field'};

		print "|-\n|";

		if( $primary_key_members{$name} ) {
			print "<b><font color=\"green\">$name</font></b> || ";
		}
		elsif( $unique_keys_members{$name} ) {
			print "<b><font color=\"blue\">$name</font></b> || ";
		}
		else {
			print "$name || ";
		}
		
		print $field->{'Type'}." || ";

		# NOTE: We print all default values inside single quotes.
		# If the field is marked as 'auto_increment', we print
		# 'auto_increment' as default value.
		if( $field->{'Extra'} =~ /auto_increment/ ) {
			print "''auto_increment'' || ";
		}
		elsif( $field->{'Default'} ) {
			print "'".$field->{'Default'}."' || ";
		}
		else {
			# Empty default value - let's print empty string in single quotes.
			# Two sequential single quotes are treated by wiki as a
			# special symbol, so we should enclose them in the 'nowiki' tag. 
			print "<nowiki>''</nowiki> || ";
		}
		
		if( $field->{'Null'} eq "YES" ) {
			print "Yes || ";
		}
		else {
			print "No || ";
		}
		
		print "\n";
	}
	
	print "|}\n\n";
}


